"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Content = void 0;
const transfer_1 = require("./transfer");
const counter_1 = require("../funcs/counter");
const unit_1 = require("./unit");
class Content {
    constructor(transfer) {
        this.transfer = transfer;
        this.counter = new counter_1.Counter();
        this.title = '';
        this.config = unit_1.dftConfig;
        this.configString = '';
        if (transfer.length === 0) {
            transfer.push([]);
        }
    }
    init() {
        this.updateConfig();
        this.counter.update(this.transfer, this.config);
        this.updateTitle();
    }
    //get
    toString() {
        return this.transfer.toString();
    }
    getConfig(limit = 100) {
        const out = new unit_1.Config();
        const transfer = this.transfer;
        let node = transfer.pointer.next;
        for (let i = 0; i < Math.min(transfer.length, limit); i++) {
            const line = node.data;
            node = node.next;
            if (line.length === 0)
                continue;
            const first = line[0];
            if (first.type !== 'unit')
                continue;
            const tag = first.tag;
            const options = first.options;
            if (options.object.isConfig === true && tag !== 'json') {
                out.setOptions(tag, options);
                out.addParam(tag, '__', first.transfer);
                out.setParams(tag, first.params);
            }
        }
        return out;
    }
    getContext() {
        const counter = this.counter;
        return {
            config: this.config,
            title: this.title,
            headingInfos: counter.headingInfos,
            labelSet: counter.labelSet,
            labelToIndexInfo: counter.labelToIndexInfo,
            labelToRefInfos: counter.labelToRefInfos,
            irInfos: counter.irInfos,
            indexInfos: counter.indexInfos,
            refInfos: counter.refInfos,
            keyToLabel: counter.keyToLabel
        };
    }
    //to
    static fromString(string) {
        return new Content(transfer_1.Transfer.fromString(string));
    }
    updateTitle() {
        const string = this.transfer.getFirstFirstSingleLineString('title').trim();
        this.title = string === '' ? 'untitled' : string;
    }
    updateConfig() {
        const config = this.getConfig();
        const configString = config.toString();
        if (configString !== this.configString) {
            this.config = new unit_1.Config();
            this.config.assign(unit_1.dftConfig);
            this.config.assign(config);
            this.configString = configString;
        }
    }
}
exports.Content = Content;
//# sourceMappingURL=content.js.map