"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Transfer = void 0;
const linkedList_1 = require("../wheels/linkedList");
const json_1 = require("./json");
const unit_1 = require("./unit");
const char_1 = require("./char");
class Transfer extends linkedList_1.DoublyLinkedList {
    //position
    static clonePosition(position) {
        return { lineIndex: position.lineIndex, inlineIndex: position.inlineIndex, right: position.right };
    }
    static rangePosition(position) {
        return {
            start: Transfer.clonePosition(position),
            end: Transfer.clonePosition(position)
        };
    }
    static cloneRange(range) {
        return {
            start: Transfer.clonePosition(range.start),
            end: Transfer.clonePosition(range.end)
        };
    }
    static comparePosition(start, end) {
        if (start.lineIndex < end.lineIndex)
            return -1;
        if (start.lineIndex > end.lineIndex)
            return 1;
        if (start.inlineIndex < end.inlineIndex)
            return -1;
        if (start.inlineIndex > end.inlineIndex)
            return 1;
        return 0;
    }
    static isCollapse(range) {
        if (this.comparePosition(range.start, range.end) === 0)
            return true;
        return false;
    }
    checkPosition(position, strict) {
        if (position.lineIndex < 0
            || position.lineIndex >= this.length
            || position.inlineIndex < 0)
            return false;
        if (strict)
            return position.inlineIndex < this.findNode(position.lineIndex).data.length;
        return position.inlineIndex <= this.findNode(position.lineIndex).data.length;
    }
    checkRange(range) {
        if (!this.checkPosition(range.start))
            return false;
        if (!this.checkPosition(range.end))
            return false;
        if (Transfer.comparePosition(range.start, range.end) > 0)
            return false;
        return true;
    }
    isEmpty() {
        if (this.length > 1)
            return false;
        if (this.length === 0)
            return true;
        const line = this.pointer.next.data;
        return line.length === 0;
    }
    left(position) {
        if (!this.checkPosition(position))
            return false;
        if (position.inlineIndex > 0)
            return { lineIndex: position.lineIndex, inlineIndex: position.inlineIndex - 1 };
        if (position.lineIndex === 0)
            return { lineIndex: 0, inlineIndex: 0 };
        return { lineIndex: position.lineIndex - 1, inlineIndex: this.findNode(position.lineIndex - 1).data.length };
    }
    right(position) {
        if (!this.checkPosition(position))
            return false;
        const length = this.findNode(position.lineIndex).data.length;
        if (position.inlineIndex < length)
            return { lineIndex: position.lineIndex, inlineIndex: position.inlineIndex + 1 };
        if (position.lineIndex === this.length - 1)
            return { lineIndex: position.lineIndex, inlineIndex: position.inlineIndex };
        return { lineIndex: position.lineIndex + 1, inlineIndex: 0 };
    }
    up(position) {
        if (!this.checkPosition(position))
            return false;
        if (position.lineIndex === 0)
            return position;
        const previous = this.findNode(position.lineIndex - 1).data;
        if (previous.length < position.inlineIndex)
            return { lineIndex: position.lineIndex - 1, inlineIndex: previous.length };
        return { lineIndex: position.lineIndex - 1, inlineIndex: position.inlineIndex };
    }
    down(position) {
        if (!this.checkPosition(position))
            return false;
        if (position.lineIndex === (this.length - 1))
            return position;
        const next = this.findNode(position.lineIndex + 1).data;
        if (next.length < position.inlineIndex)
            return { lineIndex: position.lineIndex + 1, inlineIndex: next.length };
        return { lineIndex: position.lineIndex + 1, inlineIndex: position.inlineIndex };
    }
    leftFirstBrace(position, limit = 100) {
        if (!this.checkPosition(position))
            return false;
        let node = this.pointer.next;
        for (let i = 0; i < position.lineIndex; i++) {
            node = node.next;
        }
        const line = node.data;
        node = node.previous;
        for (let j = position.inlineIndex - 1; j >= 0; j--) {
            const temp = line[j];
            if (temp.type !== 'unit' || temp.tag !== 'json')
                continue;
            if (temp.options.object._ === '{')
                return { lineIndex: position.lineIndex, inlineIndex: j };
            if (temp.options.object._ === '}')
                return false;
        }
        for (let i = position.lineIndex - 1; i >= Math.max(0, position.lineIndex - limit); i--) {
            const line = node.data;
            node = node.previous;
            for (let j = line.length - 1; j >= 0; j--) {
                const temp = line[j];
                if (temp.type !== 'unit' || temp.tag !== 'json')
                    continue;
                if (temp.options.object._ === '{')
                    return { lineIndex: i, inlineIndex: j };
                if (temp.options.object._ === '}')
                    return false;
            }
        }
        return false;
    }
    rightFirstBrace(position, limit = 100) {
        if (!this.checkPosition(position))
            return false;
        let node = this.pointer.next;
        for (let i = 0; i < position.lineIndex; i++) {
            node = node.next;
        }
        const line = node.data;
        node = node.next;
        for (let j = position.inlineIndex; j < line.length; j++) {
            const temp = line[j];
            if (temp.type !== 'unit' || temp.tag !== 'json')
                continue;
            if (temp.options.object._ === '}')
                return { lineIndex: position.lineIndex, inlineIndex: j + 1 };
            if (temp.options.object._ === '{')
                return false;
        }
        for (let i = 1 + position.lineIndex; i < Math.min(this.length, 1 + limit + position.lineIndex); i++) {
            const line = node.data;
            node = node.next;
            for (let j = 0; j < line.length; j++) {
                const temp = line[j];
                if (temp.type !== 'unit' || temp.tag !== 'json')
                    continue;
                if (temp.options.object._ === '}')
                    return { lineIndex: i, inlineIndex: j + 1 };
                if (temp.options.object._ === '{')
                    return false;
            }
        }
        return false;
    }
    leftBrace(position, count = 0, limit = 100) {
        if (!this.checkPosition(position))
            return false;
        let node = this.pointer.next;
        for (let i = 0; i < position.lineIndex; i++) {
            node = node.next;
        }
        const line = node.data;
        node = node.previous;
        for (let j = position.inlineIndex - 1; j >= 0; j--) {
            const temp = line[j];
            if (temp.type !== 'unit' || temp.tag !== 'json')
                continue;
            if (count === 0 && temp.options.object._ === '{')
                return { lineIndex: position.lineIndex, inlineIndex: j };
            if (temp.options.object._ === '{')
                count++;
            else if (temp.options.object._ === '}')
                count--;
        }
        for (let i = position.lineIndex - 1; i >= Math.max(0, position.lineIndex - limit); i--) {
            const line = node.data;
            node = node.previous;
            for (let j = line.length - 1; j >= 0; j--) {
                const temp = line[j];
                if (temp.type !== 'unit' || temp.tag !== 'json')
                    continue;
                if (count === 0 && temp.options.object._ === '{')
                    return { lineIndex: i, inlineIndex: j };
                if (temp.options.object._ === '{')
                    count++;
                else if (temp.options.object._ === '}')
                    count--;
            }
        }
        return false;
    }
    rightBrace(position, count = 0, limit = 100) {
        if (!this.checkPosition(position))
            return false;
        let node = this.pointer.next;
        for (let i = 0; i < position.lineIndex; i++) {
            node = node.next;
        }
        const line = node.data;
        node = node.next;
        for (let j = position.inlineIndex; j < line.length; j++) {
            const temp = line[j];
            if (temp.type !== 'unit' || temp.tag !== 'json')
                continue;
            if (count === 0 && temp.options.object._ === '}')
                return { lineIndex: position.lineIndex, inlineIndex: j + 1 };
            if (temp.options.object._ === '{')
                count++;
            else if (temp.options.object._ === '}')
                count--;
        }
        for (let i = 1 + position.lineIndex; i < Math.min(this.length, 1 + limit + position.lineIndex); i++) {
            const line = node.data;
            node = node.next;
            for (let j = 0; j < line.length; j++) {
                const temp = line[j];
                if (temp.type !== 'unit' || temp.tag !== 'json')
                    continue;
                if (count === 0 && temp.options.object._ === '}')
                    return { lineIndex: i, inlineIndex: j + 1 };
                if (temp.options.object._ === '{')
                    count++;
                else if (temp.options.object._ === '}')
                    count--;
            }
        }
        return false;
    }
    previousUnit(position, limit = 100) {
        if (!this.checkPosition(position))
            return false;
        let node = this.pointer.next;
        for (let i = 0; i < position.lineIndex; i++) {
            node = node.next;
        }
        const line = node.data;
        node = node.previous;
        for (let j = position.inlineIndex - 1; j >= 0; j--) {
            const temp = line[j];
            if (temp.type === 'unit') {
                return { lineIndex: position.lineIndex, inlineIndex: j };
            }
        }
        for (let i = position.lineIndex - 1; i >= Math.max(0, position.lineIndex - limit); i--) {
            const line = node.data;
            node = node.previous;
            for (let j = line.length - 1; j >= 0; j--) {
                const temp = line[j];
                if (temp.type === 'unit') {
                    return { lineIndex: i, inlineIndex: j };
                }
            }
        }
        return false;
    }
    nextUnit(position, limit = 100) {
        if (!this.checkPosition(position))
            return false;
        let node = this.pointer.next;
        for (let i = 0; i < position.lineIndex; i++) {
            node = node.next;
        }
        const line = node.data;
        node = node.next;
        for (let j = position.inlineIndex; j < line.length; j++) {
            const temp = line[j];
            if (temp.type === 'unit') {
                return { lineIndex: position.lineIndex, inlineIndex: j };
            }
        }
        for (let i = 1 + position.lineIndex; i < Math.min(this.length, 1 + limit + position.lineIndex); i++) {
            const line = node.data;
            node = node.next;
            for (let j = 0; j < line.length; j++) {
                const temp = line[j];
                if (temp.type === 'unit') {
                    return { lineIndex: i, inlineIndex: j };
                }
            }
        }
        return false;
    }
    getEndRange() {
        return Transfer.rangePosition({ lineIndex: this.length - 1, inlineIndex: this.length === 0 ? 0 : this.pointer.previous.data.length });
    }
    //inline
    get(position) {
        if (position.lineIndex < 0)
            return false;
        if (position.inlineIndex < 0)
            return false;
        if (position.lineIndex >= this.length)
            return false;
        const line = this.findNode(position.lineIndex).data;
        if (position.inlineIndex >= line.length)
            return false;
        return line[position.inlineIndex];
    }
    getFirstFirsts(tag, limit = 100) {
        const units = [];
        let node = this.pointer.next;
        for (let i = 0; i < Math.min(this.length, limit); i++) {
            const line = node.data;
            node = node.next;
            if (line.length === 0)
                continue;
            const first = line[0];
            if (first.type === 'unit' && first.tag === tag)
                units.push(first);
        }
        return units;
    }
    getFirstFirst(tag, limit = 100) {
        let node = this.pointer.next;
        for (let i = 0; i < Math.min(this.length, limit); i++) {
            const line = node.data;
            node = node.next;
            if (line.length === 0)
                continue;
            const first = line[0];
            if (first.type === 'unit' && first.tag === tag)
                return first;
        }
        return false;
    }
    hasGlobalUnit(config) {
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                if (inline.type !== 'unit')
                    continue;
                const { tag } = inline;
                if (inline.options.object.isConfig === true && tag !== 'json'
                    || config.getOption(tag, 'global') === true)
                    return true;
            }
        }
        return false;
    }
    //simple json
    getUnitSimpleJSONs() {
        const array = [];
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                if (inline.type !== 'unit' || inline.tag !== 'json')
                    continue;
                const json = inline.options.clone();
                delete json.object._;
                array.push(json);
            }
        }
        return array;
    }
    getStringSimpleJSONs() {
        return json_1.SimpleJSON.jsons(this.toSingleLinePlainString());
    }
    getSimpleJSONs() {
        return this.getStringSimpleJSONs().concat(this.getUnitSimpleJSONs());
    }
    getUnitSimpleJSON() {
        return json_1.SimpleJSON.merge(this.getUnitSimpleJSONs());
    }
    getStringSimpleJSON() {
        return json_1.SimpleJSON.merge(this.getStringSimpleJSONs());
    }
    getSimpleJSON() {
        return json_1.SimpleJSON.merge(this.getSimpleJSONs());
    }
    //param
    getParams() {
        const params = new unit_1.Params();
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                if (inline.type !== 'unit' || inline.tag !== 'param')
                    continue;
                const key = inline.options.object.key;
                if (typeof key !== 'string' || key.length === 0)
                    continue;
                params.object[key] = inline.transfer;
            }
        }
        return params;
    }
    //json
    static lineToJSON(line) {
        let string = '';
        const array = [];
        for (let i = 0; i < line.length; i++) {
            const inline = line[i];
            if (inline.type === 'char') {
                string += inline.string;
                continue;
            }
            if (string !== '')
                array.push(string);
            string = '';
            array.push(inline.toJSON());
        }
        if (string !== '')
            array.push(string);
        if (array.length === 1)
            return array[0];
        return array;
    }
    static jsonToLine(json) {
        const line = [];
        for (let i = 0; i < json.length; i++) {
            const temp = json[i];
            if (typeof temp === 'string') {
                const array = char_1.Char.lines(temp);
                if (array.length === 0)
                    continue;
                for (let i = 0; i < array.length; i++) {
                    line.push(...array[i]);
                }
                continue;
            }
            if (typeof temp !== 'object' || Array.isArray(temp))
                continue;
            const span = unit_1.Unit.fromJSON(temp);
            line.push(span);
        }
        return line;
    }
    static jsonToLines(json) {
        const lines = [];
        for (let i = 0; i < json.length; i++) {
            let any = json[i];
            if (typeof any === 'string') {
                lines.push(...char_1.Char.lines(any));
                continue;
            }
            if (!Array.isArray(any))
                any = [any];
            lines.push(Transfer.jsonToLine(any));
        }
        return lines;
    }
    static stringToLines(string) {
        const contentStr = string.replace(/\r/g, '').split('\nhistory\n', 2)[0].trim();
        const out = json_1.parse('[' + contentStr + ']');
        if (!Array.isArray(out))
            return char_1.Char.lines(string);
        const lines = Transfer.jsonToLines(out);
        if (lines.length === 0 && string !== '[]')
            return char_1.Char.lines(string);
        return lines;
    }
    static fromString(string) {
        return new Transfer(this.stringToLines(string));
    }
    static fromPlainString(string) {
        return new Transfer(char_1.Char.lines(string));
    }
    static fromJSON(json) {
        const lines = Transfer.jsonToLines(json);
        return new Transfer(lines);
    }
    static fromJSONOrString(json) {
        if (typeof json === 'string')
            json = [json];
        return Transfer.fromJSON(json);
    }
    toJSON() {
        const array = [];
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            const json = Transfer.lineToJSON(line);
            array.push(json);
        }
        return array;
    }
    toJSONOrString() {
        const json = this.toJSON();
        if (json.length === 1) {
            const line = json[0];
            if (typeof line === 'string')
                return line;
        }
        return json;
    }
    //string
    getFirstFirstStrings(tag, limit) {
        const units = this.getFirstFirsts(tag, limit);
        return units.map(unit => unit.transfer.toPlainString());
    }
    getFirstFirstString(tag, limit) {
        const unit = this.getFirstFirst(tag, limit);
        if (unit === false)
            return '';
        return unit.transfer.toPlainString();
    }
    getFirstFirstSingleLineString(tag, limit) {
        const unit = this.getFirstFirst(tag, limit);
        if (unit === false)
            return '';
        return unit.transfer.toSingleLinePlainString();
    }
    toPlainString() {
        let string = '';
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            if (i > 0)
                string += '\n';
            const line = node.data;
            node = node.next;
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                if (inline.type === 'unit') {
                    string += inline.transfer.toPlainString();
                    continue;
                }
                string += inline.string;
            }
        }
        return string;
    }
    toSingleLinePlainString() {
        let string = '';
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                if (inline.type === 'unit') {
                    string += inline.transfer.toSingleLinePlainString();
                    continue;
                }
                string += inline.string;
            }
        }
        return string;
    }
    toString() {
        return this.toJSON().map(value => json_1.stringify(value, 'arrayInObject')).join('\n');
    }
    //stream
    toStringStream() {
        return this.toStream().map((data) => __awaiter(this, void 0, void 0, function* () { return json_1.stringify(Transfer.lineToJSON(data), 'arrayInObject'); }));
    }
    //basic process
    push(line) {
        line = line.slice(0);
        this.insertDataBefore(line, this.pointer);
        return;
    }
    concat(lines) {
        lines = Transfer.cloneLines(lines);
        if (this.length === 0) {
            for (let i = 0; i < lines.length; i++) {
                this.push(lines[i]);
            }
            return;
        }
        const end = this.pointer.previous.data;
        if (lines.length === 0)
            return;
        const start = lines[0];
        for (let i = 0; i < start.length; i++) {
            end.push(start[i]);
        }
        for (let i = 1; i < lines.length; i++) {
            this.push(lines[i]);
        }
        return;
    }
    //reversible step
    add(lineIndex, lines) {
        lines = Transfer.cloneLines(lines);
        if (lineIndex < 0
            || lineIndex > this.length)
            return false;
        this.insertArray(lineIndex, lines);
        return true;
    }
    reduce(lineIndex, length) {
        if (lineIndex < 0
            || lineIndex + length > this.length)
            return false;
        this.deleteRange(lineIndex, length);
        return true;
    }
    insert(position, inlines) {
        inlines = inlines.slice(0);
        if (position.lineIndex < 0
            || position.inlineIndex < 0
            || position.lineIndex >= this.length)
            return false;
        const line = this.findNode(position.lineIndex).data;
        if (position.inlineIndex > line.length)
            return false;
        line.splice(position.inlineIndex, 0, ...inlines);
        return true;
    }
    delete(position, length) {
        if (position.lineIndex < 0
            || position.inlineIndex < 0
            || position.lineIndex >= this.length)
            return false;
        const line = this.findNode(position.lineIndex).data;
        if (position.inlineIndex + length > line.length)
            return false;
        line.splice(position.inlineIndex, length);
        return true;
    }
    //copy
    static cloneLines(lines) {
        lines = lines.slice(0);
        for (let i = 0; i < lines.length; i++) {
            const line = lines[i];
            lines[i] = line.slice(0);
        }
        return lines;
    }
    static deepCloneLines(lines) {
        const newLines = [];
        for (let i = 0; i < lines.length; i++) {
            const line = lines[i];
            const newLine = [];
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                newLine.push(inline.clone());
            }
            newLines.push(newLine);
        }
        return newLines;
    }
    copy(range) {
        if (!this.checkRange(range))
            return false;
        const transfer = new Transfer();
        if (range.start.lineIndex === range.end.lineIndex) {
            if (range.start.inlineIndex === range.end.inlineIndex) {
                transfer.push([]);
                return transfer;
            }
            const line = this.findNode(range.start.lineIndex).data;
            transfer.push(line.slice(range.start.inlineIndex, range.end.inlineIndex));
            return transfer;
        }
        let node = this.findNode(range.start.lineIndex);
        const start = node.data;
        node = node.next;
        transfer.push(start.slice(range.start.inlineIndex));
        for (let i = range.start.lineIndex + 1; i < range.end.lineIndex; i++) {
            const line = node.data;
            node = node.next;
            transfer.push(line.slice(0));
        }
        const end = node.data;
        transfer.push(end.slice(0, range.end.inlineIndex));
        return transfer;
    }
    deepCopy(range) {
        const transfer = this.copy(range);
        if (transfer === false)
            return false;
        let node = transfer.pointer.next;
        for (let i = 0; i < transfer.length; i++) {
            const line = node.data;
            node = node.next;
            for (let i = 0; i < line.length; i++) {
                line[i] = line[i].clone();
            }
        }
        return transfer;
    }
    flatten() {
        const out = new Transfer();
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            out.push([]);
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                if (inline.type !== 'unit') {
                    out.concat([[inline]]);
                    continue;
                }
                if (inline.tag === 'img') {
                    const src = inline.options.object.src;
                    if (typeof src === 'string') {
                        if (src.length > 1000) {
                            out.concat([[inline]]);
                            continue;
                        }
                    }
                }
                {
                    const json = new json_1.SimpleJSON({ _: '{' });
                    const unit = new unit_1.Unit('json', new Transfer(), json);
                    out.concat([[unit]]);
                }
                if (inline.tag === 'json') {
                    out.concat(char_1.Char.lines(inline.options.toString()));
                    {
                        const json = new json_1.SimpleJSON({ _: '}' });
                        const unit = new unit_1.Unit('json', new Transfer(), json);
                        out.concat([[unit]]);
                    }
                    continue;
                }
                out.concat(inline.transfer.clone().toArray());
                {
                    const pObject = inline.params.object;
                    const pKeys = Object.keys(pObject);
                    for (let i = 0; i < pKeys.length; i++) {
                        const key = pKeys[i];
                        const val = pObject[key];
                        const unit = new unit_1.Unit('param', val, new json_1.SimpleJSON({ key: key }));
                        out.concat([[unit]]);
                    }
                    if (!inline.options.isEmpty()) {
                        const json = inline.options.clone();
                        const unit = new unit_1.Unit('json', new Transfer(), json);
                        out.concat([[unit]]);
                    }
                    const json = new json_1.SimpleJSON({ tag: inline.tag });
                    const unit = new unit_1.Unit('json', new Transfer(), json);
                    out.concat([[unit]]);
                }
                {
                    const json = new json_1.SimpleJSON({ _: '}' });
                    const unit = new unit_1.Unit('json', new Transfer(), json);
                    out.concat([[unit]]);
                }
            }
        }
        return out;
    }
    clone() {
        const newLines = [];
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            const newLine = [];
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                newLine.push(inline);
            }
            newLines.push(newLine);
        }
        return new Transfer(newLines);
    }
    deepClone() {
        const newLines = [];
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            const newLine = [];
            for (let i = 0; i < line.length; i++) {
                newLine.push(line[i].clone());
            }
            newLines.push(newLine);
        }
        return new Transfer(newLines);
    }
    deepCloneAndClear(config) {
        const newLines = [];
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            const line = node.data;
            node = node.next;
            const newLine = [];
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                if (inline.type === 'unit' && config.getOption(inline.tag, 'clear') === true)
                    continue;
                newLine.push(inline.clone());
            }
            newLines.push(newLine);
        }
        return new Transfer(newLines);
    }
}
exports.Transfer = Transfer;
//# sourceMappingURL=transfer.js.map