"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dftConfig = exports.Unit = exports.invalidTags = exports.Config = exports.Params = void 0;
const json_1 = require("./json");
const transfer_1 = require("./transfer");
class Params {
    constructor(object = {}) {
        this.object = object;
    }
    static fromJSON(json) {
        const object = {};
        const keys = Object.keys(json);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            object[key] = transfer_1.Transfer.fromJSON(json[key]);
        }
        return new Params(object);
    }
    toJSON() {
        const out = {};
        const keys = Object.keys(this.object);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            out[key] = this.object[key].toJSON();
        }
        return out;
    }
    toString() {
        return json_1.stringify(this.toJSON());
    }
    clone() {
        const newObject = {};
        const keys = Object.keys(this.object);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            newObject[key] = this.object[key].deepClone();
        }
        return new Params(newObject);
    }
    assign(...paramss) {
        Object.assign(this.object, ...paramss.map(val => val.object));
    }
    static merge(paramss) {
        const params = new Params();
        params.assign(...paramss);
        return params;
    }
}
exports.Params = Params;
class Config {
    constructor(tagToOptions = {}, tagToParams = {}) {
        this.tagToOptions = tagToOptions;
        this.tagToParams = tagToParams;
    }
    static fromJSON(json) {
        const out = new Config();
        const tags = Object.keys(json);
        for (let i = 0; i < tags.length; i++) {
            const tag = tags[i];
            if (tag === '')
                continue;
            const object = json[tag];
            const keys = Object.keys(object);
            for (let i = 0; i < keys.length; i++) {
                const key = keys[i];
                if (key === '')
                    continue;
                const val = object[key];
                if (Array.isArray(val)) {
                    out.setParam(tag, key, transfer_1.Transfer.fromJSON(val));
                    continue;
                }
                out.setOption(tag, key, val);
            }
        }
        return out;
    }
    toJSON() {
        const pJSON = {};
        const pKeys = Object.keys(this.tagToParams);
        for (let i = 0; i < pKeys.length; i++) {
            const key = pKeys[i];
            const val = this.tagToParams[key];
            if (val === undefined)
                continue;
            pJSON[key] = val.toJSON();
        }
        const json = {};
        const keys = Object.keys(this.tagToOptions);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            const val = this.tagToOptions[key];
            if (val === undefined)
                continue;
            json[key] = val.object;
        }
        return Object.assign(json, pJSON);
    }
    toString() {
        return json_1.stringify(this.toJSON());
    }
    getOption(tag, key) {
        const options = this.tagToOptions[tag];
        if (options === undefined)
            return undefined;
        return options.object[key];
    }
    getParam(tag, key) {
        const params = this.tagToParams[tag];
        if (params === undefined)
            return undefined;
        return params.object[key];
    }
    setOptions(tag, options) {
        const oldOptions = this.tagToOptions[tag];
        if (oldOptions === undefined)
            this.tagToOptions[tag] = options;
        else
            this.tagToOptions[tag] = json_1.SimpleJSON.merge([oldOptions, options]);
    }
    setParams(tag, params) {
        const oldParams = this.tagToParams[tag];
        if (oldParams === undefined)
            this.tagToParams[tag] = params;
        else
            this.tagToParams[tag] = Params.merge([oldParams, params]);
    }
    setOption(tag, key, value = true) {
        const sj = new json_1.SimpleJSON();
        sj.object[key] = value;
        this.setOptions(tag, sj);
    }
    setParam(tag, key, value) {
        const params = new Params();
        params.object[key] = value;
        this.setParams(tag, params);
    }
    addParam(tag, key, value) {
        let params = this.tagToParams[tag];
        if (params === undefined) {
            params = new Params({ [key]: value });
            this.tagToParams[tag] = params;
            return;
        }
        const oldVal = params.object[key];
        if (oldVal === undefined) {
            params.object[key] = value;
            return;
        }
        const newVal = new transfer_1.Transfer();
        newVal.insertLinkedList(-1, oldVal);
        newVal.insertLinkedList(-1, value);
        params.object[key] = newVal;
    }
    assign(config) {
        Object.assign(this.tagToOptions, config.tagToOptions);
        Object.assign(this.tagToParams, config.tagToParams);
    }
}
exports.Config = Config;
exports.invalidTags = [
    'html',
    'base',
    'head',
    'link',
    'meta',
    'style',
    'title',
    'body',
    'noscript',
    'script',
    'acronym',
    'applet',
    'basefont',
    'bgsound',
    'big',
    'blink',
    'center',
    'content',
    'dir',
    'font',
    'frame',
    'frameset',
    'hgroup',
    'image',
    'keygen',
    'marquee',
    'menuitem',
    'nobr',
    'noembed',
    'noframes',
    'plaintext',
    'rb',
    'rtc',
    'shadow',
    'spacer',
    'strike',
    'tt',
    'xmp',
];
class Unit {
    constructor(tag, transfer = new transfer_1.Transfer(), options = new json_1.SimpleJSON(), params = new Params()) {
        this.tag = tag;
        this.transfer = transfer;
        this.options = options;
        this.params = params;
        this.type = 'unit';
        this.marks = new json_1.SimpleJSON();
        this.parsing = 0;
    }
    clone() {
        const newUnit = new Unit(this.tag, this.transfer.deepClone(), this.options.clone(), this.params.clone());
        if (this.origin !== undefined)
            newUnit.origin = this.origin;
        else
            newUnit.origin = this;
        return newUnit;
    }
    static fromJSON(json) {
        let tag = 'div';
        let transfer = new transfer_1.Transfer();
        const options = new json_1.SimpleJSON();
        const params = new Params();
        const object = options.object;
        const pObject = params.object;
        const keys = Object.keys(json);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            if (key === '')
                continue;
            const value = json[key];
            if (key === '__') {
                if (typeof value === 'string' || Array.isArray(value)) {
                    tag = 'katex';
                    transfer = transfer_1.Transfer.fromJSONOrString(value);
                }
                continue;
            }
            if (Array.isArray(value)) {
                transfer = transfer_1.Transfer.fromJSON(value);
                tag = key;
                continue;
            }
            if (typeof value === 'object') {
                const transfer = value.__;
                if (typeof transfer === 'string' || Array.isArray(transfer))
                    pObject[key] = transfer_1.Transfer.fromJSONOrString(transfer);
                continue;
            }
            if (typeof value === 'string' || typeof value === 'number' || typeof value === 'boolean') {
                object[key] = value;
            }
        }
        return new Unit(tag, transfer, options, params);
    }
    toJSON() {
        const object = this.options.object;
        const pObject = this.params.object;
        const transfer = this.transfer;
        const out = {};
        const keys = Object.keys(object);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            out[key] = object[key];
        }
        const pKeys = Object.keys(pObject);
        for (let i = 0; i < pKeys.length; i++) {
            const key = pKeys[i];
            out[key] = { __: pObject[key].toJSONOrString() };
        }
        const tag = this.tag;
        if (tag === 'katex') {
            out.__ = transfer.toJSONOrString();
            return out;
        }
        if (tag !== 'div' || !transfer.isEmpty())
            out[tag] = transfer.toJSON();
        return out;
    }
}
exports.Unit = Unit;
exports.dftConfig = Config.fromJSON({
    //basic units
    contents: {
        empty: true,
        passive: true
    },
    css: {
        erase: true,
        global: true
    },
    json: {
        clear: true,
        empty: true,
        erase: true
    },
    orbits: {
        erase: true,
        global: true
    },
    param: {
        clear: true,
        erase: true,
        inputOptionsFirst: true
    },
    parsers: {
        erase: true,
        global: true
    },
    ref: {
        empty: true,
        inputOptionsFirst: true,
        passive: true
    },
    title: {
        global: true
    },
    //orbit units
    bib: {
        empty: true,
        inputOptionsFirst: true,
        orbit: 'bib'
    },
    conjecture: {
        orbit: 'conjecture'
    },
    corollary: {
        orbit: 'corollary'
    },
    definition: {
        orbit: 'definition'
    },
    equation: {
        orbit: 'equation'
    },
    example: {
        orbit: 'example'
    },
    exercise: {
        orbit: 'exercise'
    },
    figure: {
        orbit: 'figure'
    },
    heading: {
        orbit: 'heading'
    },
    index: {
        empty: true,
        inputOptionsFirst: true,
        orbit: 'index'
    },
    lemma: {
        orbit: 'lemma'
    },
    notation: {
        orbit: 'notation'
    },
    proposition: {
        orbit: 'proposition'
    },
    remark: {
        orbit: 'remark'
    },
    theorem: {
        orbit: 'theorem'
    },
    //html elements
    //text content
    div: {
        inputOptionsFirst: true
    },
    dl: {
        inlineInside: true,
    },
    ol: {
        inlineInside: true,
    },
    ul: {
        inlineInside: true,
    },
    //inline text semantics
    a: {
        inlineInside: true,
    },
    abbr: {
        inlineInside: true,
    },
    b: {
        inlineInside: true,
    },
    bdi: {
        inlineInside: true,
    },
    bdo: {
        inlineInside: true,
    },
    br: {
        inlineInside: true,
    },
    cite: {
        inlineInside: true,
    },
    data: {
        inlineInside: true,
    },
    dfn: {
        inlineInside: true,
    },
    em: {
        inlineInside: true,
    },
    i: {
        inlineInside: true,
    },
    kbd: {
        inlineInside: true,
    },
    mark: {
        inlineInside: true,
    },
    q: {
        inlineInside: true,
    },
    rb: {
        inlineInside: true,
    },
    rp: {
        inlineInside: true,
    },
    rt: {
        inlineInside: true,
    },
    rtc: {
        inlineInside: true,
    },
    ruby: {
        inlineInside: true,
    },
    s: {
        inlineInside: true,
    },
    samp: {
        inlineInside: true,
    },
    small: {
        inlineInside: true,
    },
    span: {
        inputOptionsFirst: true,
        inlineInside: true,
    },
    strong: {
        inlineInside: true,
    },
    sub: {
        inlineInside: true,
    },
    sup: {
        inlineInside: true,
    },
    time: {
        inlineInside: true,
    },
    u: {
        inlineInside: true,
    },
    var: {
        inlineInside: true,
    },
    wbr: {
        inlineInside: true,
    },
    //demarcating edits
    del: {
        inlineInside: true,
    },
    ins: {
        inlineInside: true,
    },
    //table content
    col: {
        inlineInside: true,
    },
    colgroup: {
        inlineInside: true,
    },
    table: {
        inlineInside: true,
    },
    tbody: {
        inlineInside: true,
    },
    tfoot: {
        inlineInside: true,
    },
    thead: {
        inlineInside: true,
    },
    tr: {
        inlineInside: true,
    },
    //forms
    datalist: {
        inlineInside: true,
    },
    fieldset: {
        inlineInside: true,
    },
    label: {
        inlineInside: true,
    },
    optgroup: {
        inlineInside: true,
    },
    output: {
        inlineInside: true,
    },
    select: {
        inlineInside: true,
    },
    //interactive elements
    details: {
        inlineInside: true,
    },
    dialog: {
        inlineInside: true,
    },
    menu: {
        inlineInside: true,
    },
    //web Components
    slot: {
        inlineInside: true,
    },
    template: {
        inlineInside: true,
    },
});
//# sourceMappingURL=unit.js.map