"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Counter = void 0;
const json_1 = require("../base/json");
const globalCounterTags = [
    'orbits'
];
class Sheet {
    constructor() {
        this.headings = [];
        this.orbits = {};
    }
    add(level, orbit) {
        if (orbit === 'heading') {
            if (this.headings.length < level)
                for (let i = this.headings.length; i < level; i++) {
                    this.headings.push(0);
                }
            else {
                for (let i = level; i < this.headings.length; i++) {
                    this.headings[i] = 0;
                }
                const keys = Object.keys(this.orbits);
                for (let i = 0; i < keys.length; i++) {
                    const key = keys[i];
                    const value = this.orbits[key];
                    if (value === undefined)
                        continue;
                    if (value.length < level)
                        continue;
                    for (let i = level; i < value.length; i++) {
                        value[i] = 0;
                    }
                }
            }
            this.headings[level - 1]++;
            return this.headings.slice(0, level);
        }
        let value = this.orbits[orbit];
        if (value === undefined) {
            value = [];
            this.orbits[orbit] = value;
        }
        if (value.length < level)
            for (let i = value.length; i < level; i++) {
                value.push(0);
            }
        value[level - 1]++;
        if (this.headings.length < level - 1)
            for (let i = this.headings.length; i < level - 1; i++) {
                this.headings.push(0);
            }
        const temp = this.headings.slice(0, level - 1);
        temp.push(value[level - 1]);
        return temp;
    }
    clear() {
        this.headings = [];
        this.orbits = {};
    }
}
class Counter {
    constructor() {
        this.sheet = new Sheet();
        this.max = 0;
        this.headingInfos = [];
        this.indexInfos = [];
        this.refInfos = [];
        this.irInfos = [];
        this.dynamicUnitss = [];
        this.labelSet = {};
        this.labelToIndexInfo = {};
        this.labelToRefInfos = {};
        this.keyToLabel = {};
        this.orbitToLevel = {};
    }
    //check
    static checkInitiative(transfer) {
        let node = transfer.pointer.next;
        for (let i = 0; i < transfer.length; i++) {
            const line = node.data;
            node = node.next;
            for (let i = 0; i < line.length; i++) {
                const inline = line[i];
                if (inline.type !== 'unit')
                    continue;
                const tag = inline.tag;
                if (tag === 'json')
                    continue;
                if (inline.marks.object.static === true)
                    continue;
                if (globalCounterTags.indexOf(tag) !== -1)
                    return true;
                const label = inline.options.object.label;
                if (typeof label === 'string'
                    && label !== '')
                    return true;
                if (Counter.checkInitiative(inline.transfer))
                    return true;
                const pObject = inline.params.object;
                const pKeys = Object.keys(pObject);
                for (let i = 0; i < pKeys.length; i++) {
                    const key = pKeys[i];
                    const val = pObject[key];
                    if (Counter.checkInitiative(val))
                        return true;
                }
                inline.marks.object.static = true;
            }
        }
        return false;
    }
    //get
    getLabel(indStr, orbit) {
        if (indStr === '' || orbit === '')
            return false;
        const key = orbit + ' ' + indStr;
        const label = this.keyToLabel[key];
        if (label === undefined)
            return false;
        return label;
    }
    //process
    basicallyScanOrbits(json) {
        this.orbitToLevel = {};
        const keys = Object.keys(json);
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            if (key === '')
                continue;
            const value = json[key];
            if (typeof value !== 'number'
                || value <= 0
                || value % 1 !== 0)
                continue;
            this.orbitToLevel[key] = value;
        }
    }
    scanOrbits(transfer) {
        this.basicallyScanOrbits(json_1.SimpleJSON.merge(json_1.SimpleJSON.jsons(transfer.getFirstFirstStrings('orbits').join('\n'))).object);
    }
    basicallyScan(positions, units, transfer, config) {
        const labelSet = this.labelSet;
        const labelToIndex = this.labelToIndexInfo;
        const labelToRefInfos = this.labelToRefInfos;
        const orbitToLevel = this.orbitToLevel;
        const keyToLabel = this.keyToLabel;
        let hasDynamicUnit = false;
        let node = transfer.pointer.next;
        for (let i = 0; i < transfer.length; i++) {
            const line = node.data;
            node = node.next;
            for (let j = 0; j < line.length; j++) {
                const inline = line[j];
                if (inline.type !== 'unit')
                    continue;
                const tag = inline.tag;
                if (tag === 'json')
                    continue;
                const pObject = inline.params.object;
                const newPositions = positions.concat({ lineIndex: i, inlineIndex: j });
                const newUnits = units.concat([inline]);
                const position = newPositions[0];
                const label = inline.options.object.label;
                if (config.getOption(tag, 'passive') === true) {
                    hasDynamicUnit = true;
                    if (typeof label === 'string' && label !== '') {
                        const refInfo = {
                            type: 'ref',
                            units: newUnits,
                            unit: inline,
                            transfer: inline.transfer,
                            label: label,
                            position: position
                        };
                        let refInfos = labelToRefInfos[label];
                        if (refInfos === undefined) {
                            refInfos = [];
                            labelToRefInfos[label] = refInfos;
                        }
                        refInfos.push(refInfo);
                        this.refInfos.push(refInfo);
                        this.irInfos.push(refInfo);
                    }
                }
                else if (typeof label === 'string' && label !== '') {
                    hasDynamicUnit = true;
                    if (labelToIndex[label] === undefined) {
                        let orbit;
                        const tempOrbit = inline.options.object.orbit;
                        if (typeof tempOrbit === 'string' && tempOrbit !== '')
                            orbit = tempOrbit;
                        else {
                            const defaultOrbit = config.getOption(tag, 'orbit');
                            if (typeof defaultOrbit === 'string' && defaultOrbit !== '')
                                orbit = defaultOrbit;
                            else
                                orbit = 'heading';
                        }
                        let level;
                        const tempLevel = inline.options.object.level;
                        if (orbit === 'heading') {
                            if (typeof tempLevel !== 'number'
                                || tempLevel <= 0
                                || tempLevel % 1 !== 0)
                                level = 1;
                            else
                                level = tempLevel;
                        }
                        else {
                            const tempLevel = orbitToLevel[orbit];
                            if (tempLevel === undefined)
                                level = 1;
                            else
                                level = tempLevel;
                        }
                        const index = this.sheet.add(level, orbit);
                        const indStr = index.join('.');
                        const key = orbit + ' ' + indStr;
                        const indexInfo = {
                            type: 'index',
                            units: newUnits,
                            unit: inline,
                            transfer: inline.transfer,
                            label: label,
                            level: level,
                            index: index,
                            indStr: indStr,
                            position: position,
                            orbit: orbit
                        };
                        labelSet[label] = true;
                        labelToIndex[label] = indexInfo;
                        keyToLabel[key] = label;
                        this.indexInfos.push(indexInfo);
                        this.irInfos.push(indexInfo);
                        const number = Number(label);
                        if (!isNaN(number) && number > this.max)
                            this.max = number;
                        if (orbit === 'heading') {
                            this.headingInfos.push(indexInfo);
                        }
                    }
                }
                let result = this.basicallyScan(newPositions, newUnits, inline.transfer, config);
                const pKeys = Object.keys(pObject);
                for (let i = 0; i < pKeys.length; i++) {
                    const key = pKeys[i];
                    const val = pObject[key];
                    result = result || this.basicallyScan(newPositions, newUnits, val, config);
                }
                if (!result && hasDynamicUnit)
                    this.dynamicUnitss.push(newUnits);
            }
        }
        return hasDynamicUnit;
    }
    scan(transfer, config) {
        this.sheet.clear();
        this.max = 0;
        this.headingInfos = [];
        this.indexInfos = [];
        this.refInfos = [];
        this.irInfos = [];
        this.dynamicUnitss = [];
        this.labelSet = {};
        this.labelToIndexInfo = {};
        this.labelToRefInfos = {};
        this.keyToLabel = {};
        this.basicallyScan([], [], transfer, config);
    }
    update(transfer, config) {
        this.scanOrbits(transfer);
        this.scan(transfer, config);
    }
    replaceLabels(avoid, start) {
        const labelSet = this.labelSet;
        const labelToIndexInfo = this.labelToIndexInfo;
        const labelToRefInfos = this.labelToRefInfos;
        start = Math.max(start, this.max + 1);
        let keys = Object.keys(labelToIndexInfo);
        for (let i = 0; i < keys.length; i++) {
            const oldLabel = keys[i];
            if (avoid[oldLabel] !== true)
                continue;
            const indexInfo = labelToIndexInfo[oldLabel];
            if (indexInfo === undefined)
                continue;
            const newLabel = start.toString();
            this.max = start;
            start++;
            labelSet[oldLabel] = false;
            labelSet[newLabel] = true;
            const units = indexInfo.units;
            const unit = units[units.length - 1];
            unit.options.object.label = newLabel;
            labelToIndexInfo[oldLabel] = undefined;
            labelToIndexInfo[newLabel] = indexInfo;
            const refInfos = labelToRefInfos[oldLabel];
            if (refInfos === undefined)
                continue;
            for (let i = 0; i < refInfos.length; i++) {
                const refInfo = refInfos[i];
                const units = refInfo.units;
                const unit = units[units.length - 1];
                unit.options.object.label = newLabel;
            }
            labelToRefInfos[oldLabel] = undefined;
            labelToRefInfos[newLabel] = refInfos;
        }
    }
}
exports.Counter = Counter;
//# sourceMappingURL=counter.js.map