"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.extractLabelsWithIndex = exports.extractLabelsWithTag = void 0;
const ston = require("ston");
const stdn = require("stdn");
function extractLabelsWithTag(string) {
    const result = stdn.parse(string);
    if (result === undefined) {
        return [];
    }
    const out = [];
    for (let i = 0; i < result.length; i++) {
        const line = result[i];
        for (let i = 0; i < line.length; i++) {
            const unit = line[i];
            if (typeof unit === 'string') {
                continue;
            }
            const { label } = unit.options;
            if (typeof label !== 'string'
                || label === '') {
                continue;
            }
            out.push({
                value: label,
                tag: unit.tag
            });
        }
    }
    return out;
}
exports.extractLabelsWithTag = extractLabelsWithTag;
function extractLabelsWithIndex(string) {
    const result = ston.parseWithIndex('[' + string + ']', -1);
    if (result === undefined
        || !Array.isArray(result.value)) {
        return [];
    }
    return extractLabelsWithIndexFromSTONArrayValueWithIndex(result.value);
}
exports.extractLabelsWithIndex = extractLabelsWithIndex;
function extractLabelsWithIndexFromSTONArrayValueWithIndex(array) {
    const out = [];
    for (let i = 0; i < array.length; i++) {
        const { value } = array[i];
        if (typeof value !== 'object') {
            continue;
        }
        if (Array.isArray(value)) {
            out.push(...extractLabelsWithIndexFromSTONArrayValueWithIndex(value));
            continue;
        }
        const { label } = value;
        if (label === undefined
            || typeof label.value !== 'string'
            || label.value === '') {
            continue;
        }
        out.push({ value: label.value, index: label.index });
    }
    return out;
}
//# sourceMappingURL=label.js.map