"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = void 0;
const http = require("http");
const url_1 = require("url");
const vscode = require("vscode");
function createServer(baseURI) {
    return __awaiter(this, void 0, void 0, function* () {
        const server = http.createServer((req, res) => __awaiter(this, void 0, void 0, function* () {
            normal: {
                if (req.method !== 'GET' || req.url === undefined) {
                    break normal;
                }
                const url = new url_1.URL(req.url, 'http://localhost');
                const pathname = decodeURIComponent(url.pathname.slice(1));
                const result = pathname.match(/\.(\w+)$/);
                if (result === null) {
                    break normal;
                }
                let type = result[1];
                if (type === 'js') {
                    type = 'text/javascript';
                }
                else if (type === 'html') {
                    type = 'text/html';
                }
                else if (type === 'css') {
                    type = 'text/css';
                }
                else if (type === 'gif') {
                    type = 'image/gif';
                }
                else if (type === 'jpg' || type === 'jpeg') {
                    type = 'image/jpeg';
                }
                else if (type === 'png') {
                    type = 'image/png';
                }
                else if (type === 'svg') {
                    type = 'image/svg+xml';
                }
                else if (type === 'webp') {
                    type = 'image/webp';
                }
                else {
                    type = 'text/plain';
                }
                type += ';charset=utf-8';
                try {
                    const uri = vscode.Uri.joinPath(baseURI, pathname);
                    const data = yield vscode.workspace.fs.readFile(uri);
                    res.setHeader("Content-Type", type);
                    res.end(Buffer.from(data.buffer));
                    return;
                }
                catch (err) {
                }
            }
            res.statusCode = 400;
            res.end();
        }));
        const config = vscode.workspace.getConfiguration('stLang.preview.port');
        let tmpMin = config.get('min');
        if (typeof tmpMin !== 'number' || !isFinite(tmpMin) || tmpMin % 1 !== 0 || tmpMin < 8081) {
            tmpMin = 8081;
        }
        const min = tmpMin;
        let tmpMax = config.get('max');
        if (typeof tmpMax !== 'number' || !isFinite(tmpMax) || tmpMax % 1 !== 0 || tmpMax < min) {
            tmpMax = min;
        }
        const max = tmpMax;
        const info = {
            server: server,
            port: min,
        };
        yield new Promise(resolve => {
            server.on('error', e => {
                info.port++;
                if (server.listening) {
                    server.close();
                }
                if (info.port > max) {
                    vscode.window.showWarningMessage('Fail to start server\n' + e.message);
                    resolve(0);
                    return;
                }
                server.listen(info.port);
            });
            server.on('listening', () => {
                vscode.window.showInformationMessage('Listening on ' + info.port);
                resolve(0);
            });
            setTimeout(() => {
                resolve(0);
            }, 10000);
            server.listen(info.port);
        });
        return info;
    });
}
exports.createServer = createServer;
//# sourceMappingURL=server.js.map