"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DoublyLinkedList = exports.SinglyLinkedQueue = void 0;
const stream_1 = require("./stream");
class SinglyLinkedQueue {
    constructor() {
        this.start = {
            next: false
        };
        this.end = this.start;
    }
    push(data) {
        const node = {
            data: data,
            next: false
        };
        this.end.next = node;
        this.end = node;
    }
}
exports.SinglyLinkedQueue = SinglyLinkedQueue;
class DoublyLinkedList {
    constructor(array = []) {
        this.length = 0;
        this.pointer = {};
        this.pointer.previous = this.pointer;
        this.pointer.next = this.pointer;
        for (let i = 0; i < array.length; i++) {
            this.insertDataBefore(array[i], this.pointer);
        }
    }
    insertDataBefore(data, before) {
        const node = {
            previous: before.previous,
            next: before,
            data: data
        };
        before.previous = node;
        node.previous.next = node;
        this.length++;
    }
    normalizeIndex(index, allowNegative) {
        const period = this.length + 1;
        if (index < 0 || index >= period)
            index = index % period;
        if (index < 0)
            index += period;
        if (allowNegative && index > (-1 + period / 2)) {
            index -= period;
        }
        return index;
    }
    findNode(index) {
        index = this.normalizeIndex(index, true);
        let node = this.pointer;
        if (index >= 0)
            for (let i = -1; i < index; i++) {
                node = node.next;
            }
        else
            for (let i = -1; i > index; i--) {
                node = node.previous;
            }
        return node;
    }
    toArray() {
        const array = [];
        let node = this.pointer.next;
        for (let i = 0; i < this.length; i++) {
            array.push(node.data);
            node = node.next;
        }
        return array;
    }
    toStream() {
        const stream = new stream_1.Stream();
        new Promise(tempResolve => {
            tempResolve(true);
        }).then(() => {
            let node = this.pointer.next;
            for (let i = 0; i < this.length; i++) {
                stream.in(node.data);
                node = node.next;
            }
            stream.finish();
        });
        return stream;
    }
    sliceToArray(start, end) {
        if (start < 0)
            start = 0;
        if (end > this.length)
            end = this.length;
        if (start >= end)
            return [];
        const array = [];
        let node = this.findNode(start);
        for (let i = start; i < end; i++) {
            array.push(node.data);
            node = node.next;
        }
        return array;
    }
    sliceToStream(start, end) {
        const stream = new stream_1.Stream();
        new Promise(tempResolve => {
            tempResolve(true);
        }).then(() => {
            if (start < 0)
                start = 0;
            if (end > this.length)
                end = this.length;
            if (start >= end) {
                stream.finish();
                return;
            }
            let node = this.findNode(start);
            for (let i = start; i < end; i++) {
                stream.in(node.data);
                node = node.next;
            }
            stream.finish();
        });
        return stream;
    }
    deleteRange(start, deleteCount) {
        if (start < 0)
            start = 0;
        const last = this.length - start;
        if (deleteCount > last)
            deleteCount = last;
        if (deleteCount <= 0)
            return;
        const startNode = this.findNode(start);
        let endNode = startNode;
        for (let i = 1; i < deleteCount; i++) {
            endNode = endNode.next;
        }
        startNode.previous.next = endNode.next;
        endNode.next.previous = startNode.previous;
        this.length -= deleteCount;
    }
    insertArray(start, array) {
        const node = this.findNode(start);
        for (let i = 0; i < array.length; i++) {
            this.insertDataBefore(array[i], node);
        }
    }
    insertLinkedList(start, list) {
        const node = this.findNode(start);
        node.previous.next = list.pointer.next;
        list.pointer.next.previous = node.previous;
        node.previous = list.pointer.previous;
        list.pointer.previous.next = node;
        this.length += list.length;
    }
    insertStream(start, stream) {
        return __awaiter(this, void 0, void 0, function* () {
            const node = this.findNode(start);
            yield stream.process((data) => __awaiter(this, void 0, void 0, function* () { this.insertDataBefore(data, node); }));
        });
    }
}
exports.DoublyLinkedList = DoublyLinkedList;
//# sourceMappingURL=linkedList.js.map