"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KillableLock = exports.Lock = void 0;
class Lock {
    constructor() {
        this.queue = [];
    }
    get(wait = 10000) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield new Promise((resolve) => {
                const symbol = Symbol();
                const profile = {
                    symbol: symbol,
                    listen: () => {
                        resolve(symbol);
                    },
                    failed: false
                };
                if (wait > 0)
                    setTimeout(() => {
                        resolve(false);
                        profile.failed = true;
                    }, wait);
                this.queue.push(profile);
                if (this.queue[0].symbol === symbol) {
                    resolve(symbol);
                }
            });
            return result;
        });
    }
    release(symbol) {
        if (this.queue.length === 0 || this.queue[0].symbol !== symbol)
            return;
        this.queue.shift();
        while (this.queue.length > 0 && this.queue[0].failed) {
            this.queue.shift();
        }
        if (this.queue.length > 0)
            this.queue[0].listen();
        return;
    }
}
exports.Lock = Lock;
class KillableLock {
    constructor() {
        this.busy = false;
        this.lock = new Lock();
        this.killLock = new Lock();
        this.killing = false;
        this.killedListen = () => { };
        this.killingListen = () => __awaiter(this, void 0, void 0, function* () { });
        this.queue = [];
    }
    get(wait = 10000) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const symbol = Symbol();
                const profile = {
                    symbol: symbol,
                    listen: (killing) => {
                        if (killing) {
                            resolve(false);
                            return;
                        }
                        resolve(symbol);
                        this.busy = true;
                    },
                    failed: false
                };
                if (wait > 0)
                    setTimeout(() => {
                        resolve(false);
                        profile.failed = true;
                    }, wait);
                const result = yield this.lock.get(-1);
                if (this.killing) {
                    resolve(false);
                    this.lock.release(result);
                    return;
                }
                this.queue.push(profile);
                if (this.queue[0].symbol === symbol) {
                    resolve(symbol);
                    this.busy = true;
                }
                this.lock.release(result);
            }));
            return result;
        });
    }
    release(symbol) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.lock.get(-1);
            if (this.queue.length === 0 || this.queue[0].symbol !== symbol) {
                this.lock.release(result);
                return;
            }
            this.queue.shift();
            while (this.queue.length > 0 && this.queue[0].failed) {
                this.queue.shift();
            }
            this.busy = false;
            if (this.killing) {
                for (let i = 0; i < this.queue.length; i++) {
                    this.queue[i].listen(true);
                }
                this.queue = [];
                this.killedListen();
            }
            else {
                if (this.queue.length > 0)
                    this.queue[0].listen(false);
            }
            this.lock.release(result);
            return;
        });
    }
    kill(wait = 10000) {
        return __awaiter(this, void 0, void 0, function* () {
            const result0 = yield this.killLock.get(-1);
            const result1 = yield new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const result2 = yield this.lock.get(-1);
                this.killing = true;
                this.killedListen = () => {
                    resolve(true);
                };
                if (this.queue.length === 0) {
                    resolve(true);
                }
                else if (wait > 0)
                    setTimeout(() => {
                        resolve(false);
                    }, wait);
                const killingListen = this.killingListen;
                this.lock.release(result2);
                yield killingListen();
            }));
            this.killLock.release(result0);
            return result1;
        });
    }
    revive() {
        return __awaiter(this, void 0, void 0, function* () {
            const result0 = yield this.killLock.get(-1);
            const result1 = yield this.lock.get(-1);
            this.killing = false;
            this.queue = [];
            this.lock.release(result1);
            this.killLock.release(result0);
        });
    }
    setKillingListen(symbol, killingListen = () => __awaiter(this, void 0, void 0, function* () { })) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield this.lock.get(-1);
            if (this.queue.length === 0 || this.queue[0].symbol !== symbol) {
                this.lock.release(result);
                return;
            }
            if (this.killing) {
                this.lock.release(result);
                yield killingListen();
                return;
            }
            this.killingListen = killingListen;
            this.lock.release(result);
            return;
        });
    }
    sleep(time) {
        return __awaiter(this, void 0, void 0, function* () {
            yield new Promise((resolve) => __awaiter(this, void 0, void 0, function* () {
                const result = yield this.get(time);
                if (result === false) {
                    resolve(true);
                    return;
                }
                yield this.setKillingListen(result, () => __awaiter(this, void 0, void 0, function* () {
                    yield this.release(result);
                    resolve(true);
                }));
                setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                    yield this.release(result);
                    resolve(true);
                }), time);
            }));
        });
    }
}
exports.KillableLock = KillableLock;
//# sourceMappingURL=lock.js.map