"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Stream = void 0;
const linkedList_1 = require("./linkedList");
class Stream {
    constructor() {
        this.stream = new linkedList_1.SinglyLinkedQueue();
        this.queue = new linkedList_1.SinglyLinkedQueue();
        this.finished = false;
    }
    has(pointer) {
        return __awaiter(this, void 0, void 0, function* () {
            if (pointer.next !== false)
                return true;
            if (this.finished)
                return false;
            return new Promise(resolve => {
                this.queue.push({
                    pointer: pointer,
                    listen: () => {
                        if (this.finished) {
                            resolve(false);
                            return;
                        }
                        resolve(true);
                    }
                });
            });
        });
    }
    in(data) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.finished)
                return false;
            this.stream.push(data);
            while (this.queue.start.next !== false) {
                const temp = this.queue.start.next.data;
                if (temp.pointer.next === false)
                    break;
                temp.listen();
                this.queue.start = this.queue.start.next;
            }
            return true;
        });
    }
    finish() {
        return __awaiter(this, void 0, void 0, function* () {
            this.finished = true;
            while (this.queue.start.next !== false) {
                this.queue.start.next.data.listen();
                this.queue.start = this.queue.start.next;
            }
            return true;
        });
    }
    process(process, ignorePast = false) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(resolve => {
                new Promise(tempResolve => {
                    tempResolve(true);
                }).then(() => __awaiter(this, void 0, void 0, function* () {
                    let pointer = ignorePast ? this.stream.end : this.stream.start;
                    while (true) {
                        if (!(yield this.has(pointer))) {
                            resolve(true);
                            return;
                        }
                        try {
                            const result = yield process(pointer.next.data);
                            if (result === true) {
                                resolve(true);
                                return;
                            }
                        }
                        catch (err) {
                            resolve(false);
                            return;
                        }
                        pointer = pointer.next;
                    }
                }));
            });
        });
    }
    map(process) {
        const stream = new Stream();
        new Promise(tempResolve => {
            tempResolve(true);
        }).then(() => __awaiter(this, void 0, void 0, function* () {
            yield this.process((data) => __awaiter(this, void 0, void 0, function* () {
                const newData = yield process(data);
                stream.in(newData);
            }));
            stream.finish();
        }));
        return stream;
    }
    concat(stream) {
        const newStream = new Stream();
        new Promise(tempResolve => {
            tempResolve(true);
        }).then(() => __awaiter(this, void 0, void 0, function* () {
            yield this.process((data) => __awaiter(this, void 0, void 0, function* () { newStream.in(data); }));
            yield stream.process((data) => __awaiter(this, void 0, void 0, function* () { newStream.in(data); }));
            newStream.finish();
        }));
        return newStream;
    }
}
exports.Stream = Stream;
//# sourceMappingURL=stream.js.map